@@ -24,15 +24,88 @@
 #include <errno.h>
 #include <unistd.h>
 #include <grp.h>
+#include <getopt.h>
 
 #include "tools-common.h"
 
+/*
+ * Display the usage
+ */
+static void usage(int status, const char *program_name)
+{
+	if (status != 0) {
+		fprintf(stderr, "Wrong input parameters,"
+			" try %s -h' for more information.\n",
+			program_name);
+	} else {
+		fprintf(stdout, "Usage: %s [-h] [-f mode] [-d mode] "\
+			"[-t <tuid>:<tgid>] [-a <agid>:<auid>] "\
+			"-g <controllers>:<path> [-g ...]\n",
+			program_name);
+		fprintf(stdout, "  -t <tuid>:<tgid>		Set "\
+			"the task permission\n");
+		fprintf(stdout, "  -a <tuid>:<tgid>		Set "\
+			"the admin permission\n");
+		fprintf(stdout, "  -g <controllers>:<path>	Control "\
+			"group which should be added\n");
+		fprintf(stdout, "  -h,--help			Display "\
+			"this help\n");
+		fprintf(stdout, "  -f, --fperm mode		Group "\
+			"file permissions\n");
+		fprintf(stdout, "  -d, --dperm mode		Group "\
+			"direrory permissions\n");
+	}
+}
+
+/* allowed mode strings are octal version: "755" */
+
+int parse_mode(char *string, mode_t *pmode, const char *program_name)
+{
+	mode_t mode = 0;
+	int pos = 0; /* position of the number iin string */
+	int i;
+	int j = 64;
+
+	while (pos < 3) {
+		if ('0' <= string[pos] && string[pos] < '8') {
+			i = (int)string[pos] - (int)'0';
+			/* parse the permission triple*/
+			mode = mode + i*j;
+			j = j / 8;
+		} else {
+			fprintf(stdout, "%s wrong mode format %s",
+				program_name, string);
+			return -1;
+		}
+		pos++;
+	}
+
+	/* the string have contains three characters */
+	if (string[pos] != '\0') {
+		fprintf(stdout, "%s wrong mode format %s",
+			program_name, string);
+		return -1;
+	}
+	*pmode = mode;
+	return 0;
+}
+
 int main(int argc, char *argv[])
 {
 	int ret = 0;
 	int i, j;
 	int c;
 
+	static struct option long_opts[] = {
+		{"help", no_argument, NULL, 'h'},
+		{"task", required_argument, NULL, 't'},
+		{"admin", required_argument, NULL, 'a'},
+		{"", required_argument, NULL, 'g'},
+		{"dperm", required_argument, NULL, 'd'},
+		{"fperm", required_argument, NULL, 'f' },
+		{0, 0, 0, 0},
+	};
+
 	/* Structure to get GID from group name */
 	struct group *grp = NULL;
 	char *grp_string = NULL;
@@ -51,23 +124,32 @@ int main(int argc, char *argv[])
 	/* approximation of max. numbers of groups that will be created */
 	int capacity = argc;
 
+	/* permission variables */
+	mode_t dir_mode = 0;
+	mode_t file_mode = 0;
+	int dirm_change = 0;
+	int filem_change = 0;
+
 	/* no parametr on input */
 	if (argc < 2) {
-		fprintf(stderr, "Usage is %s "
-			"-t <tuid>:<tgid> -a <agid>:<auid> "
-			"-g <list of controllers>:<relative path to cgroup>\n",
-			argv[0]);
+		usage(1, argv[0]);
 		return -1;
 	}
 	cgroup_list = calloc(capacity, sizeof(struct cgroup_group_spec *));
 	if (cgroup_list == NULL) {
 		fprintf(stderr, "%s: out of memory\n", argv[0]);
-		return -1;
+		ret = -1;
+		goto err;
 	}
 
 	/* parse arguments */
-	while ((c = getopt(argc, argv, "a:t:g:")) > 0) {
+	while ((c = getopt_long(argc, argv, "a:t:g:hd:f:", long_opts, NULL))
+		> 0) {
 		switch (c) {
+		case 'h':
+			usage(0, argv[0]);
+			ret = 0;
+			goto err;
 		case 'a':
 			/* set admin uid/gid */
 			if (optarg[0] == ':')
@@ -86,7 +168,8 @@ int main(int argc, char *argv[])
 					fprintf(stderr, "%s: "
 						"can't find uid of user %s.\n",
 						argv[0], pwd_string);
-					return -1;
+					ret = -1;
+					goto err;
 				}
 			}
 			if (grp_string != NULL) {
@@ -97,7 +180,8 @@ int main(int argc, char *argv[])
 					fprintf(stderr, "%s: "
 						"can't find gid of group %s.\n",
 						argv[0], grp_string);
-					return -1;
+					ret = -1;
+					goto err;
 				}
 			}
 
@@ -120,7 +204,8 @@ int main(int argc, char *argv[])
 					fprintf(stderr, "%s: "
 						"can't find uid of user %s.\n",
 						argv[0], pwd_string);
-					return -1;
+					ret = -1;
+					goto err;
 				}
 			}
 			if (grp_string != NULL) {
@@ -131,7 +216,8 @@ int main(int argc, char *argv[])
 					fprintf(stderr, "%s: "
 						"can't find gid of group %s.\n",
 						argv[0], grp_string);
-					return -1;
+					ret = -1;
+					goto err;
 				}
 			}
 			break;
@@ -142,15 +228,22 @@ int main(int argc, char *argv[])
 					"cgroup controller and path"
 					"parsing failed (%s)\n",
 					argv[0], argv[optind]);
-				return -1;
+				ret = -1;
+				goto err;
 			}
 			break;
-		default:
-			fprintf(stderr, "%s: "
-				"invalid command line option\n",
-				argv[0]);
-			return -1;
+		case 'd':
+			dirm_change = 1;
+			ret = parse_mode(optarg, &dir_mode, argv[0]);
 			break;
+		case 'f':
+			filem_change = 1;
+			ret = parse_mode(optarg, &file_mode, argv[0]);
+			break;
+		default:
+			usage(1, argv[0]);
+			ret = -1;
+			goto err;
 		}
 	}
 
@@ -217,6 +310,17 @@ int main(int argc, char *argv[])
 			cgroup_free(&cgroup);
 			goto err;
 		}
+		if (dirm_change + filem_change > 0) {
+			ret = cg_chmod_recursive(cgroup, dir_mode, dirm_change,
+				file_mode, filem_change);
+			if (ret) {
+				fprintf(stderr, "%s: can't change permission " \
+					"of cgroup %s: %s\n", argv[0],
+					cgroup->name, cgroup_strerror(ret));
+				cgroup_free(&cgroup);
+				goto err;
+			}
+		}
 		cgroup_free(&cgroup);
 	}
 err:
